local addonName, ns = ...
local _G = _G
local MODULE_NAME = 'GuildXP'
local addon = ns.addon
local module = addon:createModule(MODULE_NAME, addon.xpBarTemplate, 'AceEvent-3.0')
local setfenv = _G.setfenv
setfenv(1, module)

local min, floor, ceil = _G.min, _G.floor, _G.math.ceil
local assert = _G.assert
local tostring, string = _G.tostring, _G.string

local UnitGetGuildXP = _G.UnitGetGuildXP
local GetGuildLevel  = _G.GetGuildLevel
local QueryGuildXP   = _G.QueryGuildXP

local MAX_GUILD_LEVEL = ns.MAX_GUILD_LEVEL
local XP_PER_GUILD_LEVEL = ns.XP_PER_GUILD_LEVEL

-- start xp
function module:getStartXPMin()
    return 0
end

-- target level
function module:getTargetLevelMax()
    return MAX_GUILD_LEVEL
end

module.oldEnable = module.OnEnable
function module:OnEnable()
    self:oldEnable()
    self:RegisterEvent('GUILD_XP_UPDATE', 'Invalidate')
    -- TODO find proper events

    QueryGuildXP()
end

function module:Redraw()
    --addon:Debug("Guild XP Redraw")

    local xpOptions = self:getOptions()
    local barCache = self.cache

    local guildLevel = GetGuildLevel('player')
    local currentGlobalXP = self:getCurrentXP()
    -- update max settings if necessary
    if xpOptions.targetXP and currentGlobalXP > xpOptions.targetXP then
        xpOptions.targetXP = currentGlobalXP
    end
    local startGlobalXP = xpOptions.startXP
    local targetGlobalXP = xpOptions.targetXP or self:calcXpForLevel(MAX_GUILD_LEVEL)
    local remainingGlobalXP = self:getRemainingDailyXP()
    local currentGlobalXPRatio =   (currentGlobalXP  -startGlobalXP)/(targetGlobalXP-startGlobalXP)
    local remainingGlobalXPRatio = (remainingGlobalXP              )/(targetGlobalXP-startGlobalXP)
    -- create textures
    assert(barCache and barCache.frame)
    local frame = barCache.frame
    local right = frame:GetRight()
    local left = frame:GetLeft()
    if not (right and left) then
        self:Invalidate()
        return
    end

    -- texture for curent xp
    self:DrawPrimaryBar(currentGlobalXPRatio)

    -- texture for remaining xp
    local ratio2 = min(remainingGlobalXPRatio, 1-currentGlobalXPRatio)
    self:DrawSecondaryBar(ratio2)

    self:DrawNumbers()

    --update options table
    self:updateOptionsTable()
end

function module:getCurrentXP()
    local xpInCurrentLevel, _, _, _ = UnitGetGuildXP('player')
    local level = GetGuildLevel('player')
    return xpInCurrentLevel + self:calcXpForLevel(level)
end

function module:getRemainingDailyXP()
    local _, _, dailyXP, maxDailyXP = UnitGetGuildXP('player')
    return maxDailyXP - dailyXP
end

function module:calcXpForLevel(level)
    assert(level <= MAX_GUILD_LEVEL,"level=" .. tostring(level))
    local sum = 0
    for i=1,level-1 do
        sum = sum + XP_PER_GUILD_LEVEL[i]
    end
    local fraction = level - floor(level)
    local xpForNextLevelLocal = XP_PER_GUILD_LEVEL[floor(level)]
    if xpForNextLevelLocal then
        return sum + fraction * xpForNextLevelLocal
    else
        return sum
    end

end

function module:calcLevelForXP(xp)
    local level = 1
    while xp > self:calcXpForLevel(level+1) do
        level = level + 1
        assert(level <= MAX_GUILD_LEVEL, "level overflow")
    end
    local startXP = self:calcXpForLevel(level)
    assert(startXP <= xp,"level=" .. level .. " ,xp=" .. xp .. " ,startXP=" .. startXP)
    if startXP == xp then
        return level
    end
    local nextXP = self:calcXpForLevel(level+1)
    local fraction = (xp-startXP)/(nextXP-startXP)
    assert(0.0 <= fraction and fraction <= 1.0, "fraction=" .. fraction .. ", level=" .. level .. " ,xp=" .. xp .. " ,startXP=" .. startXP)
    return level + fraction
end

function module:fillTooltip(tooltip)
    QueryGuildXP()
    local currentGlobalXP = self:getCurrentXP()
    local targetGlobalXP = self:getOptions().targetXP or self:getTargetXPMax()
    local startGlobalXP = self:getOptions().startXP
    local hasXP = currentGlobalXP - startGlobalXP
    local deltaXP = targetGlobalXP-startGlobalXP
    local remainingGlobalXP = self:getRemainingDailyXP()
    self:addNameToTooltip(tooltip)
    self:fillTooltipWithSecond(tooltip, floor(hasXP), ceil(deltaXP), remainingGlobalXP)
end

