local addonName, ns = ...
local _G = _G
local MODULE_NAME = 'PlayerRep'
local addon = ns.addon
local module = addon:createModule(MODULE_NAME, addon.repBarTemplate, 'AceEvent-3.0')
local setfenv = _G.setfenv
setfenv(1, module)

local type = _G.type
local pcall = _G.pcall
local assert = _G.assert

local min = _G.min

local GetFactionInfoByID = _G.GetFactionInfoByID

local REP_DATA  = ns.REP_DATA
local REP_NAMES = ns.REP_NAMES

module.optionsTable.args.factionId = {
    name = "Faction",
    type = 'input',
    validate = 'checkFaction',
    get = 'getFaction',
    set = 'setFaction',
}

function module:getFaction(info)
    return self:getOptions().factionId
end

function module:setFaction(info,id)
    self:getOptions().factionId = id
    self:Invalidate()
end

function module:checkFaction(info, input)
    if (not input) or (not type(input) == 'number') then
       return "Not a number"
    end
    -- do a protected call, if it suceeds this is a valid faction
    if pcall(function() GetFactionInfoByID(input) end) then
        local name = GetFactionInfoByID(input)
        addon:Debug(input, name, "is a faction")

        return true
    else
        addon:Debug(input,"is no faction")
        return "Not a valid faction"
    end
end

--function module:getFactions()
--
--end

module.oldEnable = module.OnEnable
function module:OnEnable()
    self:oldEnable()
    self:RegisterEvent('UPDATE_FACTION', 'Invalidate')
end

function module:Redraw()

    local options = self:getOptions()
    local barCache = self.cache

    local currentGlobalXP = self:getCurrentXP()

    -- update max settings if necessary
    if options.targetXP and currentGlobalXP > options.targetXP then
       options.targetXP = currentGlobalXP
    end
    if options.startXP and currentGlobalXP < options.startXP then
        options.startXP = currentGlobalXP
    end

    local startGlobalXP  = options.startXP  or currentGlobalXP
    local targetGlobalXP = options.targetXP or currentGlobalXP

    local currentGlobalXPRatio =   (currentGlobalXP-startGlobalXP)/(targetGlobalXP-startGlobalXP)

    assert(barCache and barCache.frame)
    local frame = barCache.frame
    local right = frame:GetRight()
    local left = frame:GetLeft()
    if not (right and left) then
        self:Invalidate()
        return
    end
    local width = frame:GetRight() - frame:GetLeft()
    -- texture for curent xp
    self:DrawPrimaryBar(currentGlobalXPRatio)

    -- textures and numbers for levels
    self:drawReputationNumbers()

    --update options table
    self:updateOptionsTable()
end

function module:getCurrentXP()
    local factionId = self:getOptions().factionId
    if not factionId then return 0 end
    local isOk = pcall(function() GetFactionInfoByID(factionId) end)
    if not isOk then
        return 0
    end
    local name, description, standingID, barMin, barMax,
          barValue, atWarWith, canToggleAtWar, isHeader, isCollapsed,
          hasRep, isWatched, isChild = GetFactionInfoByID(factionId)
    assert(barMax - barMin == REP_DATA[standingID], "barMin=" .. barMin .. " barMax=" .. barMax)
    local levelRep = self:calcXpForLevel(standingID)
    assert(levelRep == barMin)
    local sum = barValue
    addon:Debug(name,standingID,levelRep, "value",barValue,"min", barMin,"max",barMax, sum)
    return sum
end

function module:fillTooltip(tooltip)
    local currentGlobalXP = self:getCurrentXP()
    local targetGlobalXP = self:getOptions().targetXP or self:getTargetXPMax()
    local startGlobalXP = self:getOptions().startXP
    local deltaXP = targetGlobalXP-startGlobalXP
    local value = currentGlobalXP-startGlobalXP
    self:addNameToTooltip(tooltip)
    self:fillTooltipOnlyPrimary(tooltip, value, deltaXP)
end

--module.defaults.targetXP = module:calcXpForLevel(module:getTargetLevelMax())